from django.utils.translation import gettext_lazy as _

undefined = {
    "code": "undefined",
    "en": _("undefined exception code"),
}

tag_100_1 = {
    "code": "tag_100_1",
    "en": _("Tag already exists. name: {name}"),
}

tag_100_2 = {
    "code": "tag_100_2",
    "en": _("Tag not found: {name}"),
}

tag_100_3 = {
    "code": "tag_100_3",
    "en": _("Key not found."),
}

tag_100_4 = {
    "code": "tag_100_4",
    "en": _("Object ids not found."),
}
